/*****************************************************************************
  *                                                                           
  *  Copyright (c) 2013 www.der-frickler.net                   
  *                                                                           
  *                                                                           
  *  This library is free software; you can redistribute it and/or modify it  
  *  under the terms of the GNU Lesser General Public License as published    
  *  by the Free Software Foundation; either version 2 of the License, or     
  *  (at your option) any later version.                                      
  *                                                                           
  *  This software is distributed in the hope that it will be useful, but     
  *  WITHOUT ANY WARRANTY; without even the implied warranty of               
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        
  *  Lesser General Public License for more details.                          
  *                                                                           
  *  You should have received a copy of the GNU Lesser General Public         
  *  License along with this library; if not, write to the Free Software      
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA  
  *                                                                           
  *****************************************************************************
  *                                                                           
  *  NunchukFunk.ino
  *  RC Sender basierend auf einem Wii Nnuchuk Controller
  *  erzeugt ein 8ch PPM signal bei dem ch 1/2 über die ACC Sensoren udn ch3/4 
  *  über den eingebauten Joystick gesteuert werden. 
  *  der Z-Button toggelt ACC an/aus, C-Button schaltet Kanal 5 high/low
  *                                                                           
  *****************************************************************************/

#include <Wire.h>
#include "ArduinoNunchuk.h"

//////////////////////CONFIGURATION///////////////////////////////
#define chanel_number 8  //set the number of chanels
#define PPM_FrLen 20000  //set the PPM frame length in microseconds (1ms = 1000µs)
#define PPM_PulseLen 400  //set the pulse length
#define onState 0  //set polarity of the pulses: 1 is positive, 0 is negative
#define sigPin 9  //set PPM signal output pin on the arduino
//////////////////////////////////////////////////////////////////

#define XMIX 1

#define CENTER_1 1480
#define CENTER_2 1500
#define CENTER_3 1500
#define CENTER_4 1500
#define CENTER_5 1000
#define CENTER_6 1000
#define CENTER_7 1000
#define CENTER_8 1000

//#include <Servo.h> 
// Servo myservo; 

ArduinoNunchuk nunchuk = ArduinoNunchuk();

//#define TWI_FREQ 400000L

int ppm[chanel_number];

boolean locked = true, armed = false;
int lastZButton = 0, lastCButton = 0;;
int buttonZState, buttonCState;               // the current reading from the input pin
float accX, accY;

// the following variables are long's because the time, measured in miliseconds,
// will quickly become a bigger number than can be stored in an int.
long lastZDebounceTime = 0, lastCDebounceTime = 0;  // the last time the output pin was toggled
long debounceDelay = 50;    // the debounce time; increase if the output flickers


void setup()
{
  Serial.begin(115200);
  
    Serial.println("Startup");
  // power nunchuck from Arduino Pins:
  pinMode(A3, OUTPUT);  
  digitalWrite(A3, HIGH);
  pinMode(A2, OUTPUT);  
  digitalWrite(A2, LOW);
  
  nunchuk.init();
    Serial.println("Nunchuk done");
  
  //myservo.attach(9); 

  ppm[0] = CENTER_1;
  ppm[1] = CENTER_2;
  ppm[2] = CENTER_3;
  ppm[3] = CENTER_4;
  ppm[4] = CENTER_5;
  ppm[5] = CENTER_6;
  ppm[6] = CENTER_7;
  ppm[7] = CENTER_8;

  pinMode(sigPin, OUTPUT);
  digitalWrite(sigPin, !onState);  //set the PPM signal pin to the default state (off)
  
  cli();
  TCCR1A = 0; // set entire TCCR1 register to 0
  TCCR1B = 0;
  
  OCR1A = 100;  // compare match register, change this
  TCCR1B |= (1 << WGM12);  // turn on CTC mode
  TCCR1B |= (1 << CS11);  // 8 prescaler: 0,5 microseconds at 16mhz
  TIMSK1 |= (1 << OCIE1A); // enable timer compare interrupt
  sei();

 Serial.println("PPM Started");
 
 nunchuk.update();
 accX = nunchuk.accelX;
 accY = nunchuk.accelY;
}

void loop()
{
  nunchuk.update();

//  Serial.print(nunchuk.analogX, DEC);
//  Serial.print(' ');
//  Serial.print(nunchuk.analogY, DEC);
//  Serial.print(' ');
//  Serial.print(nunchuk.accelX, DEC);
//  Serial.print(' ');
//  Serial.print(nunchuk.accelY, DEC);
//  Serial.print(' ');
//  Serial.print(nunchuk.accelZ, DEC);
//  Serial.print(' ');
//  Serial.print(nunchuk.zButton, DEC);
//  Serial.print(' ');
//  Serial.print(nunchuk.cButton, DEC);
//  Serial.println();
  
  // Debouce and toggle Z-Button
  if (nunchuk.zButton != lastZButton) {
    lastZDebounceTime = millis();
  } 
  if ((millis() - lastZDebounceTime) > debounceDelay) {
    if (nunchuk.zButton != buttonZState) {
      buttonZState = nunchuk.zButton;
      if (buttonZState == HIGH) {
        locked = !locked;
      }
    }
  }
  
  // Debouce and toggle C-Button
  if (nunchuk.cButton != lastCButton) {
    lastCDebounceTime = millis();
  } 
  if ((millis() - lastCDebounceTime) > debounceDelay) {
    if (nunchuk.cButton != buttonCState) {
      buttonCState = nunchuk.cButton;
      if (buttonCState == HIGH) {
        armed = !armed;
      }
    }
  }

  // save the reading.  Next time through the loop,
  lastZButton = nunchuk.zButton;
  lastCButton = nunchuk.cButton;
  
  digitalWrite(13, !locked);
  
  // smoothen ACC readings
  accX = 0.6*accX + 0.4*nunchuk.accelX;
  accY = 0.6*accY + 0.4*nunchuk.accelY;

  // if ACC activated
  if (!locked) {
    
    // Nick
    int ppmX = map(accX, 250, 750, -500, 500);
    // Roll
    int ppmY = map(accY, 250, 750, -500, 500);  
    
    if (XMIX) {
      ppm[0] = (ppmX+ppmY)+1500;
      ppm[1] = (ppmY-ppmX)+1500; 
    } else {
      ppm[0] = ppmX+1500;
      ppm[1] = ppmY+1500;      
    }
  } else {
    ppm[0] = CENTER_1;
    ppm[1] = CENTER_2;
  }
  ppm[0] = constrain(ppm[0], 1000,2000);
  ppm[1] = constrain(ppm[1], 1000,2000);     
  
  // Throttle
  ppm[2] = map(nunchuk.analogY, 20, 220, 1000, 2000);
  ppm[2] = constrain(ppm[2], 1000,2000);
  // Yaw
  ppm[3] = map(nunchuk.analogX, 20, 225, 1000, 2000); 
  ppm[3] = constrain(ppm[3], 1000,2000); 
  // aux 2
  ppm[5] = map(armed, 0, 1, 1000, 2000);

  Serial.print(ppm[0], DEC);
  Serial.print(' ');
  Serial.print(ppm[1], DEC);
  Serial.print(' ');
  Serial.print(ppm[2], DEC);
  Serial.print(' ');
  Serial.print(ppm[3], DEC);
  Serial.print(' ');
  Serial.print(ppm[4], DEC);
  Serial.print(' ');
  Serial.print(ppm[5], DEC);
  Serial.print(' ');
  Serial.print(ppm[6], DEC);
  Serial.print(' ');
  Serial.print(ppm[7], DEC);
  Serial.println();

}


ISR(TIMER1_COMPA_vect){  //leave this alone
  static boolean state = true;
  
  TCNT1 = 0;
  
  if(state) {  //start pulse
    digitalWrite(sigPin, onState);
    OCR1A = PPM_PulseLen * 2;
    state = false;
  }
  else{  //end pulse and calculate when to start the next pulse
    static byte cur_chan_numb;
    static unsigned int calc_rest;
  
    digitalWrite(sigPin, !onState);
    state = true;

    if(cur_chan_numb >= chanel_number){
      cur_chan_numb = 0;
      calc_rest = calc_rest + PPM_PulseLen;// 
      OCR1A = (PPM_FrLen - calc_rest) * 2;
      calc_rest = 0;
    }
    else{
      OCR1A = (ppm[cur_chan_numb] - PPM_PulseLen) * 2;
      calc_rest = calc_rest + ppm[cur_chan_numb];
      cur_chan_numb++;
    }     
  }
}


